/*  TDSLOG
This program is written using Microsoft Visual C 4.1 or 1.52 and the 
National Instruments GPIB 95 Drivers.
It shows how to xfer waveforms from the TDS using SRQ handeling. And 
save the waveform data in binary and ascii files.  The binary files
can then be read using WaveStar >= ver. 1.1.
Written By Chris Skach copyright Tektronix 3/25/97
Problems: TDS200 2CM module with ver <1.4 will not convert ascii file voltage
correctly.  The ymult returns the wrong value.  The TDS200 binary file can not 
be read by wavestar ver <=1.1.2.  This will be fixed in wavestar. ver 1.1.3
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>
#include <sys\timeb.h>

/*#include "decl.h"	*/	/* use this for < c4.0 compiler */		

#include <windows.h>		 /*use these for > c4.0 compiler*/ 
#include "decl-32.h" 

#define MAX_CNT 100         /* Max number of acquisitions */
#define WAV_SRC 9           /* Scope sources available */
FILE *wvfile;				/* waveform log file pointer */
char *wave[MAX_CNT];        /* pointer for waveform array*/
int lgth_num;               /* var for record number */

void getWfms(int scope,int brd,int acqCnt,int chCnt,char *wave); /*Get triggered waveforms*/
int gpibWrite( int scope, char *cmd); /* gpib read/write functions   */
int gpibRead( int scope, char *cmd, int cnt);
void gpiberr(char *msg);       /* gpib error functions   */

void main(){
	char source[WAV_SRC][17];  /* Array for waveform source */
	char encdg []="SRIBINARY"; /* set encoding here */
	char src[17];              /* String for scope source*/
	char recordl[10];          /* String for record length from Scope */
	char preamb [4][500];      /* waveform preamble */
	char cmd[80];              /* String for GPIB commands*/
	char id[80];			   /* scope id */
	char isfname[13];		   /* binary waveform log filename */
	char datname[13];		   /* binary waveform log filename */
	char time[12], date[12];   /* date & time */
	char *ptr;				   /* temp pointer */
	int brd;                   /* handle for gpib board: GPIB0 */
    int scope;                 /* address of scope */
	int model=0;               /* 1 = TDS200 series or TDS 800, 0 = other */
	int length=500;            /* init record length */
	int width=1;               /* data width */
	int start=1;               /* data start */
	int stop=500;              /* init data stop */
	int acqCnt=1;              /* Max acquistions default is 1*/
	int intSrc;                /* integer source select */
	int chNum=1;               /* to do length * 4 if all 4 ch's*/
	int i,pt,j,k;                 /* Counters */
	float twob;				   /* used for two byte data */
	int pt_off[4];			   /* preamble info */
	float yoff[4], ymult[4], yzero[4], xincr[4], xzero[4];  /* preamble info */
	float t0,t1;               /* Timer stuff*/
	time_t  sec;
	unsigned short  milli;
	struct  timeb    tbuff;

	memset( cmd, 0, 80 ); /* initialize command string to all nulls */ 
	memset( id, 0, 80 ); /* initialize command string to all nulls */ 

	
/*
 *  Open a Handle to GPIB0 and check for errors.
 */
	if( ((brd = ibfind("GPIB0")) < 0)){
		gpiberr("ibfind Error: Unable to find board!");
		exit(0);
	}

/*
 *  Assign a unique identifier to the device and store in the variable
 *  scope.  IBDEV opens an available device and assigns it to access GPIB0
 *  with a primary address of 1, a secondary address of 0, a timeout of
 *  10 seconds, the END message enabled, and the EOS mode disabled.
 *  If scope is less than zero, call GPIBERR with an error message.
 */
	if( (scope = ibdev(0, 1, 0, T10s, 1, 0)) < 0){
		gpiberr("ibdev Error: Unable to find device!");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (brd, 0);
		exit(0);
	}

/*
 *  Clear the device and check for errors.
 */
	if ((ibclr(scope) & ERR) ||
	  (ibsre(brd,0) & ERR)){
		gpiberr("ibclr/ibsre Error: Unable to clear device/board!");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(0);
	}
	/*scope = ibfind("DEV1");   added for 16 bit srq to work */	   
	if (gpibWrite( scope, "ID?") & ERR){
		gpiberr("ibwrt Error: ID?");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
		
	if(ibrd( scope, id, 80 ) & ERR){
		gpiberr("ibrd Error: ID?");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
	if (strstr(id,"TDS 2")|| strstr(id,"TDS 8"))
		model =2;
	if (strstr(id,"TDS 3"))
		model =3;

	
    printf("\n                  %s\n",id); /*print scope id & version*/
	printf ("                         TDS AUTO LOG PROGRAM\n");
	printf ("                           Version 2.00 3-97\n");
	printf ("                             Tektronix Inc.\n");
	printf (" \n");
	printf ("                     SETUP SCOPE TO ACQUIRE SIGNAL\n");
	printf ("                     PRESS THE ENTER KEY WHEN READY\n");
	getchar();
	

	/* Fill Scope Source Selections Array */
	strcpy(source[1],"CH1");
	strcpy(source[2],"CH2");
	strcpy(source[3],"CH3");
	strcpy(source[4],"CH4");
	strcpy(source[5],"MATH1");
	strcpy(source[6],"MATH2");
	strcpy(source[7],"REF1");
	strcpy(source[8],"CH1,CH2,CH3,CH4");
    

	/* Select Scope Waveform Source */
	do{
		/*List all Waveform Sources available*/
		printf ("\tTDS WAVEFORM SOURCE SELECTION TABLE \n");
		for (i=1;i < (WAV_SRC);i++)
			printf("\t[%d] %s\n",i,source[i]);
		printf ("\nENTER WAVEFORM LOG SOURCE<> ");
		scanf("%s",cmd);  /*read input */
		intSrc = atoi(cmd);
	}while (intSrc <= 0 || intSrc >= WAV_SRC); /* wrong input */
	strcpy (src,source[intSrc]);  /* used for scope source command */
	if (intSrc == 8 && model != 2 && model !=3 ) /* check for 2 channel scopes */
		chNum=4;
	else 
		chNum=1;

	do {
		printf ("SELECT 1 or 2 Byte Data <> ");
		scanf ( "%d",&width);
	}while (width < 0 || width > 2);
	    
	sprintf (cmd,"DATA:WIDTH %d",width);    /* select width */
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup DATA:WIDTH");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}

	
	/* get number of acquisitions */
	do {
		printf ("SELECT UP TO %d ACQUISITIONS TO BE ACQUIRED<>",(MAX_CNT/chNum));
		scanf ( "%d",&acqCnt);

	}while (acqCnt < 0 || acqCnt > (MAX_CNT/chNum));

	acqCnt = (acqCnt * chNum);
	printf ("The Scope is Setting\n");

	
	sprintf (cmd,"DATA:ENCDG %s",encdg);    /* select encoding */
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup DATA:ENCDG");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}


	sprintf (cmd,"DATA:SOURCE %s",src);     /* select source */
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup Data Source");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
	sprintf (cmd,"DATA:START %d",start);      /* set start*/
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup Data Start");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
    if (chNum == 1)
		sprintf (cmd,"SELECT:%s ON",src);      /* select source */
	else
		sprintf (cmd,"SELECT:CH1 ON;CH2 ON;CH3 ON;CH4 ON"); 
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Select Channel");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
	
	/**********GET LENGTH OF RECORD ***********/

	strcpy (cmd,"HEADER OFF;HOR:RECO?");        /*get record length*/
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup HOR:RECO?");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
	if(ibrd(scope,recordl,10)& ERR){      /*read value from scope*/
		gpiberr("ibrd Error: Unable to Read Record Length");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
	length = atoi(recordl);
	if(width == 2)
		length = length * 2; /* 2 byte data */
	stop = length;
	
	sprintf (cmd,"DATA:STOP %d",stop);      /* set stop*/
	if((gpibWrite( scope, cmd )) & ERR){
		gpiberr("ibwrt Error: Unable to Setup Stop");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}

	/***************ALLOCATE MEMORY FOR WAVEFORM*************/
	printf("Allocating Memory\n");
	for(i=0;i<=acqCnt;i++){
		if(( wave[i] = (char *)malloc((size_t) (length) )) == NULL){
			printf ("OUT OF MEMORY \n");
			exit(0);
		}
		memset(wave[i],'\0',length); /* null out array */
	}

		
	/* SETUP SCOPE FOR OPC and SRQ use LIMIT instead of SEQ for limit test*/ 
	if((gpibWrite( scope, "DESE 1;*ESE 1;*SRE 32") & ERR)||
		(gpibWrite(scope, "ACQUIRE:STOPAFTER SEQ") & ERR) ||
       	(gpibWrite( scope, "HEADER OFF") & ERR))
    {
		gpiberr("ibwrt Error: Unable to Setup waveform parameters");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
    printf ("                     THE SCOPE IS READY TO ACQUIRE\n");
    printf ("                PRESS THE ENTER KEY WHEN READY TO START\n");
	getchar();
	getchar();


	/* Get start time */
	ftime (&tbuff);
	milli = tbuff.millitm;
	sec = tbuff.time;
	t0=(float)((sec*1000) + milli);
	
	/*printf ("The Scope is Waiting for a Trigger\n");*/
	for(i=0; i<acqCnt; i++){        /* get count acqs */
		for (j=0;j<chNum;j++)		/* used for 4 channel capture */
			getWfms(scope,brd,acqCnt,j,wave[i+j]);
		
		i=i+(j-1);

	}

	/* get stop time */
	ftime (&tbuff);
	milli = tbuff.millitm;
	sec = tbuff.time;
	t1 =(float)((sec*1000) + milli);
	if (acqCnt >= 5){
		/************** PRINT TRANSFER RATE ************/
		printf("transfer rate = %.2f %d channel acquisitions/sec\n",((acqCnt/((t1 - t0)/1000))/chNum),chNum);
		printf("%d waveforms were xfered in %.2f sec\n", acqCnt,((t1 - t0)/1000));
	}


	_strdate( date );
	_strtime( time );
	ibloc(scope); /* Unlock scope Frontpanel*/
	printf ("\nDO YOU WANT TO SAVE WAVEFORMS TO A LOG FILE? Y/N ");
	scanf("%s",cmd);
	if (!strcmpi(cmd,"y")){
			/*
		* Read the waveform preamble.
		*/
		for (i=0;i<chNum;i++){
			sprintf(cmd,"DATA:SOURCE CH%d;:HEADER ON;VERBOSE ON;WFMPRE?",i+1);
			if((gpibWrite( scope, cmd)) & ERR){
				gpiberr("ibwrt Error: Unable to Setup waveform parameters");
				/*  Call the ibonl function to disable the hardware and software. */
				ibonl (scope, 0);
				ibonl (brd, 0);
				exit(1);
			}
			/* read PREAMBLE from scope */
			if(ibrd(scope,preamb[i],500)& ERR){        
				gpiberr("ibrd Error: Unable to read preamble");
				/*  Call the ibonl function to disable the hardware and software. */
				ibonl (scope, 0);
				ibonl (brd, 0);
				exit(1);
			}
			ptr = strstr(preamb[i],"\n");
			*ptr = '\0'; /*get rid of lf*/
		}


		/******SAVE Waveforms TO FILE***************/
		printf ("\nDO YOU WANT TO SAVE AS BINARY OR ASCII? B/A ");
		scanf("%s",cmd);
		if (!strcmpi(cmd,"a")){ /* ASCII FILE */
			/* if 4 channels selected get preambe from just ch1 */
			for  (i=0;i<chNum;i++){
				if (chNum!=1)
					sprintf(src,"CH%d",i+1);
				if (model!=2)
					sprintf( cmd, "DATA:SOURCE %s;:HEADER OFF;:WFMPRE:%s:YOFF?;YMULT?;YZERO?;XINCR?;PT_OFF?",src,src);
				/* tds2xx or tds8xx */
				else
					sprintf( cmd, "DATA:SOURCE %s;:HEADER OFF;:WFMPRE:%s:YOFF?;YMULT?;YZERO?;XINCR?;XZERO?",src,src);
				if(gpibWrite( scope, cmd ) & ERR)
				{
					gpiberr("ibwrt Error: WFMPRE?");
					/*  Call the ibonl function to disable the hardware and software. */
					ibonl (scope, 0);
					ibonl (brd, 0);
					exit(1);
				}
				memset( cmd, 0, 80 );   /* initialize the string buffer */
				if(ibrd( scope, cmd, 80 ) & ERR)
				{
				  gpiberr("ibrd Error: WFMPRE");
				  /*  Call the ibonl function to disable the hardware and software. */
				  ibonl (scope, 0);
				  ibonl (brd, 0);
				  exit(1);
				}
				if (model!=2) 
					sscanf( cmd, "%e;%e;%e;%e;%d;%s;%s", &yoff[i], &ymult[i],
							&yzero[i], &xincr[i], &pt_off[i] );
				/* tds2xx or tds8xx */
				else
					sscanf( cmd, "%e;%e;%e;%e;%e;%s;%s", &yoff[i], &ymult[i],
							&yzero[i], &xincr[i], &xzero[i] );
			}
			
			k=0;
			for (i=0;i<acqCnt;i++){ /* count through acquisition */
				k++;                /* used for file name */
				for (j=0;j<chNum;j++){ /*count through channels */
					sprintf (datname,"tek%d-%d.dat",k,j+1);
					if((wvfile = fopen(datname,"wb")) == NULL){
						printf("unable to open %s\n", datname);
						exit(0);
					}
					/* header info */
					printf ("\nDO YOU WANT WANT THE HEADER INFO ATTACHED TO FILE Y/N?");
					scanf("%s",cmd);
					if (!strcmpi(cmd,"y")){ /* ASCII FILE */

						fprintf(wvfile,"%s\n%s,%s\n%s\n",id,date,time,preamb[j]);
						fprintf(wvfile,":;CURVE #%d%d\n",lgth_num,length);
					}
					if (model!=2){
						/*
						* Output scaled x, y values in (Sec, Volts)
						* Time[i] = (i - PTOFF) * XINCR
						* Volts[i] = ((point value - YOFF) * YMULT) + yzero
						*/
						for (pt=0;pt<length;pt++){ /*don't read lf */
							if (width == 1){
								if (length > 15000)
									fprintf(wvfile, "%2.5e,%2.3e\n",
										(pt-pt_off[j]) * xincr[j],
										((wave[i+j][pt] - yoff[j]) * ymult[j]) + yzero[j]);
								else
									fprintf(wvfile, "%2.3e,%2.3e\n",
										(pt-pt_off[j]) * xincr[j],
										((wave[i+j][pt] - yoff[j]) * ymult[j]) + yzero[j]);
							}
							/* 2 byte data */
							else{
								twob = ((float)wave[i+j][pt] || (((float)wave[i+j][pt+1])*256));
								if (length > 15000)
									fprintf(wvfile, "%2.5e,%2.6e\n",
										((pt/2)-pt_off[j]) * xincr[j],
										((twob - yoff[i+j]) * ymult[j]) + yzero[j]);
								else
									fprintf(wvfile, "%2.3e,%2.6e\n",
										((pt/2)-pt_off[j]) * xincr[j],
										((twob - yoff[i+j]) * ymult[j]) + yzero[j]);
								pt++;
							}

						}
					}
					/* tds2xx or tds8xx 
					Time[i] = XZERO + (i * XINCR)
					Volts[i] = ((point value - YOFF) * YMULT) + YZERO
					*/
					else{
						for (pt=0;pt<length;pt++){ /*don't read lf */
							if (width == 1)
								fprintf(wvfile, "%2.3e,%2.2e\n",
									xzero[j] + (pt * xincr[j]),
									(wave[i+j][pt] - yoff[j]) * ymult[j]);
							/* 2 byte data 
							   float value = low byte + (high byte * 255)
							*/
							else{
								twob = ((float)wave[i+j][pt] || (((float)wave[i+j][pt+1])*256));
								fprintf(wvfile, "%2.3e,%2.6e\n",
									xzero[j] + ((pt/2) * xincr[j]),
									((twob - yoff[j]) * ymult[j]) + yzero[j]);
								pt++;
							}
						}
					}
					fclose(wvfile);
					printf ("Waveforms saved in %s file\n",datname);

				}
				if (j>0)
					i=i+(j-1); /* update acq counter to match channel loop */
			}
		}
		/* binary file */
		else{
			k=0;
			for (i=0;i<acqCnt;i++){ /*count through channels */
				k++;
				for (j=0;j<chNum;j++){
					sprintf (isfname,"tek%d-%d-.isf",k,j+1);
					if((wvfile = fopen(isfname,"wb")) == NULL){
						printf("unable to open %s\n", isfname);
						exit(0);
					}
					fprintf (wvfile, "%s;:CURVE #%d%d",preamb[j],lgth_num,length);
					for (pt=0;pt<=length;pt++){
						fprintf(wvfile,"%c",wave[i+j][pt]);
					}
					fclose(wvfile);
					printf ("Waveforms saved in %s file\n",isfname);
				}
				i=i+(j-1); /* update acq counter to match channel loop */

			}
		}
	}
}
/*
getWfms
This function acquires waveforms from TDS into a character array
aguments:	scope -- int gpib address for scope
		    brd --   int gpib address for gpib board
			acqCnt -- int number of acquisitions to get
			chCnt  -- number of channels to be acquired
			*wave  -- char pointer to waveform array
returns:	nothing
*/
void getWfms(int scope,int brd,int acqCnt, int chCnt, char *wave){
	int length=0;
	char recordl[10];
	char cmd[80];
	char spr,c;
	
	/* get OPC on first channel triggerd */
	if (chCnt == 0){
		/*clear out any pending SRQ's & enable SRQ*/

		if((gpibWrite( scope, "*CLS") & ERR)||
			(gpibWrite(scope, "ACQUIRE:STATE ON") & ERR) ||
			(gpibWrite( scope, "*OPC") & ERR))
		{
			gpiberr("ibwrt Error: Unable to Send *OPC");
			/*  Call the ibonl function to disable the hardware and software. */
			ibonl (scope, 0);
			ibonl (brd, 0);
			exit(1);
		}

	   	/*
       	*  Wait for the TDS to request service (RQS) or wait for    
       	*  the TDS to timeout(TIMO).  The default timeout period is 
       	*  10 seconds. RQS is detected by bit position 11 (hex 800).   
       	*  TIMO is detected by bit position 14 (hex 4000).  These status 
       	*  bits are listed under the NI-488 function IBWAIT in the 
       	*  Function Reference Manual.  If the error bit ERR or the 
       	*  timeout bit TIMO is set in IBSTA, call GPIBERR with an error 
       	*  message.
       	*/
       	ibwait (scope,TIMO|RQS);
       	if (ibsta&0x4000){    /*check ibstat for GPIB Timeout*/
			printf ("Your GPIB Timed Out before ACQ was complete\n");
			exit(0);
		}
 
     	/*
       	*  Read the TDS serial poll status byte.   If the error bit
       	* ERR is set in IBSTA, call GPIBERR with an error message.
       	*/
    	ibrsp (scope, &spr);
        if (ibsta & ERR) gpiberr("ibrsp Error");

     	/*
      	*  If the returned status byte is hex 60,dec 96, the TDS has valid data 
      	*  to send; otherwise, it has a fault condition to report.  If the 
      	*  status byte is not hex 60, call error handler with an error message.
      	*/
        if (spr != 0x60){ 			
        	printf ("ibrsp=:%d Your GPIB Recieved a SRQ from another Device\n",c);
			exit(0);
		}
 
		
	
		/**** GET WAVEFORMS ************/
		strcpy (cmd,"CURVE?");
		if((gpibWrite( scope, cmd )) & ERR){
			gpiberr("ibwrt Error: Unable to Send CURVE?");
			/*  Call the ibonl function to disable the hardware and software. */
			ibonl (scope, 0);
			ibonl (brd, 0);
			exit(1);
		}
	}
	/*
	* Read in the header information. The header includes #<x><yyy>.
	*/
	ibrd( scope, &c, 1 );      /* read in the '#' symbol */
	ibrd( scope, &c, 1 );      /* read in string length of num bytes to transfer */
	lgth_num = atoi(&c);           /* convert string to integer */
	ibrd(scope, recordl,lgth_num);  /* read in string containing num bytes to transfer */
	recordl[lgth_num]='\0';         /* force a null terminated string */
	length = atoi(recordl);          /* num bytes to transfer */
	/*
	 * Read the raw waveform data including the linefeed at the end.
	 */
	if(ibrd (scope,wave,(length+ 1)) & ERR )
	{
		gpiberr("ibrd Error: WAVEFORM READ");
		/*  Call the ibonl function to disable the hardware and software. */
		ibonl (scope, 0);
		ibonl (brd, 0);
		exit(1);
	}
}